function grid_simulation_curli_2D_2
close all; clc; clear all
% With this model, the characteristic time of curli will be estimated. The main goal for this model is to find out which factors determine the biofilm strength.
prob_turn = 0;                                                    % Probability to turn, prob = 1; only straight forward, prob = 0; completely random
colour_dif = 0.5;                                                   % Colour of curli after csgA attachment
colour_csgA = 0;                                                    % 0 for not showing csgA, e.g. 0.2 for showing csgA
number_of_small = 11;                                               % Required to make appropriate cell shape
number_of_cells = 4;                                                % Number of cells
R_max = 2000;                                                       % Maximum number of simulation steps
colour_csgB = 0.2;                                                  % Reactive center or csgB
colour_cells = 0.99;                                                % Colour of the cells
mul_factor = 2;                                                     % Factor for csgB placement
step = 22;                                                          % Factor for csg placement
step_A = 10;                                                         % Place csgA at a distance 5 from cell
i_0 = []; j_0 = [];
%% Control placement of csgA
spacing = 3;                                                        % Spacing between csgA molecules
number_of_small_csgA = 5;                                           % Spacing between csgA molecules
mul_factor_csgA = 5;                                                % Control horizontal spacing between csgA molecules

dim = 170;                                                          % Length of matrix N
cel_1_j = 50; cel_2_j = 30; cel_3_j = 130; cel_4_j = 110;
% with cel_1_j > cel_2_j and cel_3_j > cel_4_j
N = cell_build_2D(number_of_small,number_of_cells,step,...          % csgB placemnet
    colour_csgB,colour_cells,dim,mul_factor,cel_1_j, cel_2_j, cel_3_j, cel_4_j);

%% for speeding up the process
index = csgA_2D(spacing,number_of_small_csgA,number_of_cells,step_A,step,mul_factor_csgA...
    ,cel_1_j, cel_2_j, cel_3_j, cel_4_j);
i_0_p = index(:,1); j_0_p = index(:,2);                              % Vectors contain position (i,j) of csgA molecule in N matrix
total_csgA = length(i_0_p);
csgA_p_c   = total_csgA./number_of_cells;
i_0_cell_1 = i_0_p(1:csgA_p_c);
i_0_cell_2 = i_0_p(csgA_p_c+1:2.*csgA_p_c);
i_0_cell_3 = i_0_p(2.*csgA_p_c+1:3.*csgA_p_c);
i_0_cell_4 = i_0_p(3.*csgA_p_c+1:4.*csgA_p_c);
j_0_cell_1 = j_0_p(1:csgA_p_c);
j_0_cell_2 = j_0_p(csgA_p_c+1:2.*csgA_p_c);
j_0_cell_3 = j_0_p(2.*csgA_p_c+1:3.*csgA_p_c);
j_0_cell_4 = j_0_p(3.*csgA_p_c+1:4.*csgA_p_c);

%% Constants
D_protein = 1e-10;                                                  % Diffusion constant [m^2/s]
grid_length = 2e-9;                                                 % Grid length[m]
tau = grid_length.^2 ./ D_protein;                                  % [s]
R_counter = 0;                                                      % Initial amount of simulation steps
tic
%% Define position of the cells (cell do not move)
cell_parts = nnz(N==colour_csgB);                                   % Count number of cell parts
%% Possible direction (= up, right, down, left)
state_changes = [1 0; 0 1; -1 0; 0 -1];                             % Possible directions, (= up, right, down, left)
%% Move the csgA throughout the matrix
teller = zeros(dim);
class = zeros(dim);
prob = fliplr(linspace(0,1,10));

while R_counter < R_max
    R_counter = R_counter + 1;
    if mod(R_counter,10) == 0 || R_counter == 1
        k = randsample(30,2);                                       % Select for an inuction level of 0.2% (w/v) rhamnose induction
        %k = randsample(30,3);                                      % Select for an inuction level of 0.5% (w/v) rhamnose induction
        for i = 1:length(k)
            i_0_c_1(i) = i_0_cell_1(k(i));i_0_c_2(i) = i_0_cell_2(k(i));i_0_c_3(i) = i_0_cell_3(k(i));i_0_c_4(i) = i_0_cell_4(k(i));
            j_0_c_1(i) = j_0_cell_1(k(i));j_0_c_2(i) = j_0_cell_2(k(i));j_0_c_3(i) = j_0_cell_3(k(i));j_0_c_4(i) = j_0_cell_4(k(i));
        end
        i_0_c = [i_0_c_1,i_0_c_2,i_0_c_3,i_0_c_4]';
        j_0_c = [j_0_c_1,j_0_c_2,j_0_c_3,j_0_c_4]';
        i_0 = [i_0;i_0_c];
        j_0 = [j_0;j_0_c];
    end
    % Make a movie, and save the movie to 'grid_simulation_2D.gif'
    if mod(R_counter,50) == 0 || R_counter == 1                     % Save for every 10 steps a movieframe to 'grid_simulation_2D.gif'
        filename = 'grid_simulation_2D.gif';
        figure(1)
        imagesc(N)
        drawnow
        frame = getframe(1);
        im = frame2im(frame);
        [A,map] = rgb2ind(im,256);
        if R_counter == 1
            imwrite(A,map,filename,'gif','LoopCount',Inf,'DelayTime',1);
        else
            imwrite(A,map,filename,'gif','WriteMode'...
                ,'append','DelayTime',0.25);                        % Delay time is time between frames
        end
    end
    %% Initialization of i and j
    i_max = length(i_0);
    rand_nums = rand(i_max,R_max);                                  % Matrix of csga_max rows and R_max columns
    probab = [0.25 0.5 0.75 1];                                     % Each csgA molecule can move in each direction with the same probability
    probab = repmat(probab,i_max,1);
    selected_ind = [];
    s_0 = 0;
    for i = 1:i_max
        selected_ind(i) = find(probab(i,:)>rand_nums(i,R_counter),1,'first');
    end
    selected_ind = selected_ind';                                   % Selected index
    direction = state_changes(selected_ind,:);
    %% Periodic boundaries
    for i = 1:i_max
        i_0(i) = i_0(i) + direction(i,1);
        j_0(i) = j_0(i) + direction(i,2);
        if N(i_0(i),j_0(i))  == colour_csgB || N(i_0(i),j_0(i)) == colour_cells || N(i_0(i),j_0(i)) == colour_dif
            i_0(i) = i_0(i) - direction(i,1);
            j_0(i) = j_0(i) - direction(i,2);
        end
        if i_0(i) <= 1
            i_0(i) = i_0(i) + (dim - 2);
        end
        if i_0(i) >= dim
            i_0(i) = i_0(i) - (dim - 2);
        end
        if j_0(i) <= 1
            j_0(i) = j_0(i)+ (dim - 2);
        end
        if j_0(i) >= dim
            j_0(i) = j_0(i) - (dim - 2);
        end
    end
    for i = 1:i_max
        i = i - s_0;
        N(i_0(i),j_0(i)) = colour_csgA;
        if N(i_0(i) + 1, j_0(i)) == colour_csgB && (class(i_0(i) + 1, j_0(i)) == 1 || class(i_0(i) + 1, j_0(i)) == 0);
            if rand() <= prob(teller(i_0(i),j_0(i))+1)
                teller(i_0(i),j_0(i))= teller(i_0(i)+1,j_0(i)) + 1;
                if rand() <= prob_turn
                    class(i_0(i), j_0(i)) = 1;
                else
                    class(i_0(i), j_0(i)) = randi([3 4],1,1);
                end
                N(i_0(i),j_0(i)) = colour_csgB;
                N(i_0(i) + 1, j_0(i)) = colour_dif;
                i_0(i) = []; j_0(i) = [];
                s_0 = s_0 + 1;
            end
            continue
        end
        if N(i_0(i) - 1, j_0(i)) == colour_csgB && (class(i_0(i) - 1, j_0(i)) == 2 || class(i_0(i) - 1, j_0(i)) == 0);
            if rand() <= prob(teller(i_0(i),j_0(i))+1)
                teller(i_0(i),j_0(i))= teller(i_0(i)-1,j_0(i)) + 1;
                if rand() <= prob_turn
                    class(i_0(i), j_0(i)) = 2;
                else
                    class(i_0(i), j_0(i)) = randi([3 4],1,1);
                end
                N(i_0(i),j_0(i)) = colour_csgB;
                N(i_0(i) - 1, j_0(i)) = colour_dif;
                i_0(i) = []; j_0(i) = [];
                s_0 = s_0 + 1;
            end
            continue
        end
        if N(i_0(i), j_0(i) + 1) == colour_csgB && (class(i_0(i), j_0(i) + 1) == 3 || class(i_0(i), j_0(i) + 1) == 0);
            if rand() <= prob(teller(i_0(i),j_0(i))+1)
                teller(i_0(i),j_0(i))= teller(i_0(i),j_0(i)+1) + 1;
                if rand() <= prob_turn
                    class(i_0(i), j_0(i)) = 3;
                else
                    class(i_0(i), j_0(i)) = randi([1 2],1,1);
                end
                N(i_0(i),j_0(i)) = colour_csgB;
                N(i_0(i), j_0(i) + 1) = colour_dif;
                i_0(i) = []; j_0(i) = [];
                s_0 = s_0 + 1;
            end
            continue
        end
        if N(i_0(i), j_0(i) - 1) == colour_csgB && (class(i_0(i), j_0(i) - 1) == 4 || class(i_0(i), j_0(i) - 1) == 0);
            if rand() <= prob(teller(i_0(i),j_0(i))+1)
                teller(i_0(i),j_0(i))= teller(i_0(i),j_0(i)-1) + 1;
                if rand() <= prob_turn
                    class(i_0(i), j_0(i)) = 4;
                else
                    class(i_0(i), j_0(i)) = randi([1 2],1,1);
                end
                N(i_0(i),j_0(i)) = colour_csgB;
                N(i_0(i), j_0(i) - 1) = colour_dif;
                i_0(i)= []; j_0(i) = [];
                s_0 = s_0 + 1;
            end
            continue
        end
        
    end
end
toc
csga_max = R_max.*length(k);                                             % Maximum number of csgA units
t_fin = tau.*R_max
number_of_csga_attached = nnz(N==colour_dif);
percentage_csga_attached = (number_of_csga_attached./csga_max).*100      % Percentage of csgA attached

% figure(1);hold on; colorbar

teller(teller == 0) = [];
figure(2); hist(teller,max(teller))
xlabel('Number of csgA proteins in one curli'); ylabel('Frequency');
legend('0.2% (w/v) rhamnose induction');

average = mean(teller) 


end

function N = cell_build_2D(number_of_small,number_of_cells,step,colour_csgb,colour_cells,dim,mul_factor,cel_1_j, cel_2_j, cel_3_j, cel_4_j)
N = zeros(dim,dim);
assignin('base','N',N)
%% Make the cells
N_j = [cel_1_j cel_2_j cel_3_j cel_4_j cel_1_j cel_2_j cel_3_j cel_4_j];
N_i = [((cel_1_j + cel_2_j)./2).*ones(1,4) ((cel_3_j + cel_4_j)./2).*ones(1,4)];

for j = 1:number_of_cells
    step_j = 0;
    for i = 1:number_of_small
        step_j = step_j + 1;
        N(N_i(2*j-1) + step_j: N_i(2*j-1) + step - step_j , N_j(2*j-1) + step_j   ) = colour_cells;
        N(N_i(2*j) + step_j: N_i(2*j) + step - step_j , N_j(2*j) - step_j   ) = colour_cells;
        N(N_i(2*j-1) : N_i(2*j) + step , N_j(2*j) : N_j(2*j-1)) = colour_cells;
    end
end
%% Make the csgB
for j = 1:number_of_cells
    step_j = 0;
    for i = 1:number_of_small
        step_j = step_j + 1;
        % Upper site of cell
        N(N_i(2*j-1) + step_j - 1, N_j(2*j-1) + step_j   ) = colour_csgb;
        N(N_i(2*j) + step_j - 1, N_j(2*j) - step_j   ) = colour_csgb;
        % Lower site of cell
        N(N_i(2*j-1) - step_j + 1 + step, N_j(2*j-1) + step_j   ) = colour_csgb;
        N(N_i(2*j) - step_j + 1 + step, N_j(2*j) - step_j   ) = colour_csgb;
        % Middle part of cell
        N(N_i(2*j-1) - 1 , N_j(2*j) + mul_factor.*step_j - mul_factor) = colour_csgb;
        N(N_i(2*j-1) + 1 + step, N_j(2*j) + mul_factor.*step_j - mul_factor) = colour_csgb;
    end
end
end

function index = csgA_2D(spacing,number_of_small_csgA,number_of_cells,step_A,step,mul_factor_csgA,cel_1_j, cel_2_j, cel_3_j, cel_4_j)
N_j = [cel_1_j cel_2_j cel_3_j cel_4_j cel_1_j cel_2_j cel_3_j cel_4_j];
N_i = [((cel_1_j + cel_2_j)./2).*ones(1,4) ((cel_3_j + cel_4_j)./2).*ones(1,4)];
i_0 = []; j_0 = [];
for j = 1:number_of_cells
    step_j = 0;
    for i = 1:number_of_small_csgA
        step_j = step_j + 1;
        % Upper site of cell
        i_0_1(i) = [N_i(2*j-1) + spacing.*step_j - step_A];
        j_0_1(i) = [N_j(2*j-1) + spacing.*step_j   ];
        
        i_0_2(i) = [N_i(2*j) + spacing.*step_j - step_A];
        j_0_2(i) = [N_j(2*j) - spacing.*step_j   ];
        % Lower site of cell
        i_0_3(i) = [N_i(2*j-1) - spacing.*step_j + step_A + step];
        j_0_3(i) = [N_j(2*j-1) + spacing.*step_j];
        
        i_0_4(i) = [N_i(2*j) - spacing.*step_j + step_A + step];
        j_0_4(i) = [N_j(2*j) - spacing.*step_j   ] ;
        % Middle part of cell
        i_0_5(i) = [N_i(2*j-1) - step_A];
        j_0_5(i) = [N_j(2*j) + mul_factor_csgA.*step_j - mul_factor_csgA];
        
        i_0_6(i) = [N_i(2*j-1) + step_A + step];
        j_0_6(i) = [N_j(2*j) + mul_factor_csgA.*step_j - mul_factor_csgA] ;
    end
    i_0_0 = [i_0_1,i_0_2,i_0_3,i_0_4,i_0_5,i_0_6]';
    j_0_0 =  [j_0_1,j_0_2,j_0_3,j_0_4,j_0_5,j_0_6]';
    i_0 = [i_0_0;i_0];
    j_0 = [j_0_0;j_0];
end
index = [i_0,j_0];

end





